/******************************************************************************
**
** Copyright (C) 2022 The Qt Company Ltd.
** Contact: https://www.qt.io/licensing/
**
** This file is part of the Qt Quick Ultralite module.
**
** $QT_BEGIN_LICENSE:COMM$
**
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and The Qt Company. For licensing terms
** and conditions see http://www.qt.io/terms-conditions. For further
** information use the contact form at http://www.qt.io/contact-us.
**
** $QT_END_LICENSE$
**
******************************************************************************/
#pragma once

#include <platforminterface/screen.h>

#include "fsl_dc_fb.h"
#include "fsl_mipi_dsi.h"

#include "platform_config.h"

/* Where the frame buffer is shown in the screen. */
#define DEMO_BUFFER_START_X 0U
#define DEMO_BUFFER_START_Y 0U

#define DEMO_LCDIFV2 LCDIFV2
#define DEMO_LCDIFV2_IRQn LCDIFv2_IRQn
#define DEMO_LCDIFV2_IRQHandler LCDIFv2_IRQHandler

#define DEMO_DISPLAY_CONTROLLER_ELCDIF  0
#define DEMO_DISPLAY_CONTROLLER_LCDIFV2 1

#if (USE_MIPI_PANEL == MIPI_PANEL_RK055AHD091)

  #define DEMO_PANEL_WIDTH  (720)
  #define DEMO_PANEL_HEIGHT (1280)
  #define DEMO_PANEL_NAME   "RK055AHD091"

  #define DEMO_HSW 8
  #define DEMO_HFP 32
  #define DEMO_HBP 32
  #define DEMO_VSW 2
  #define DEMO_VFP 16
  #define DEMO_VBP 14

#elif (USE_MIPI_PANEL == MIPI_PANEL_RK055IQH091)

  #define DEMO_PANEL_WIDTH  (540)
  #define DEMO_PANEL_HEIGHT (960)
  #define DEMO_PANEL_NAME   "RK055IQH091"

  #define DEMO_HSW 2
  #define DEMO_HFP 32
  #define DEMO_HBP 30
  #define DEMO_VSW 2
  #define DEMO_VFP 16
  #define DEMO_VBP 14

#elif (USE_MIPI_PANEL == MIPI_PANEL_RK055MHD091)

  #define DEMO_PANEL_WIDTH  (720)
  #define DEMO_PANEL_HEIGHT (1280)
  #define DEMO_PANEL_NAME   "RK055MHD091"

  #define DEMO_HSW 6
  #define DEMO_HFP 12
  #define DEMO_HBP 24
  #define DEMO_VSW 2
  #define DEMO_VFP 16
  #define DEMO_VBP 14

#elif (USE_MIPI_PANEL == MIPI_PANEL_RK043FN02HC)

  #define DEMO_PANEL_WIDTH  (480)
  #define DEMO_PANEL_HEIGHT (272)
  #define DEMO_PANEL_NAME   "RK043FN02HC"

  #define DEMO_HSW 2
  #define DEMO_HFP 32
  #define DEMO_HBP 30
  #define DEMO_VSW 2
  #define DEMO_VFP 16
  #define DEMO_VBP 14

#elif (USE_MIPI_PANEL == MIPI_PANEL_KD050WVTPA045)

  #define DEMO_PANEL_WIDTH  (800)
  #define DEMO_PANEL_HEIGHT (480)
  #define DEMO_PANEL_NAME   "KD050WVTPA045"

  #define DEMO_HSW 48
  #define DEMO_HFP 40
  #define DEMO_HBP 40
  #define DEMO_VSW 3
  #define DEMO_VFP 141
  #define DEMO_VBP 29

#elif (USE_MIPI_PANEL == MIPI_PANEL_NHD50800480TFATXL)

  #define DEMO_PANEL_WIDTH  (800)
  #define DEMO_PANEL_HEIGHT (480)
  #define DEMO_PANEL_NAME   "NHD50800480TFATXL"

  #define DEMO_HSW 48
  #define DEMO_HFP 40
  #define DEMO_HBP 40
  #define DEMO_VSW 3
  #define DEMO_VFP 141
  #define DEMO_VBP 29

#elif (USE_MIPI_PANEL == MIPI_PANEL_HDMI_720X480_60)

  #define DEMO_PANEL_WIDTH  (720)
  #define DEMO_PANEL_HEIGHT (480)
  #define DEMO_PANEL_NAME   "HDMI_720X480_60"

  #define DEMO_HSW 62
  #define DEMO_HBP 60
  #define DEMO_HFP 16
  #define DEMO_VSW 6
  #define DEMO_VBP 30
  #define DEMO_VFP 9

  #define DEMO_VIDEO_FREQ_KHZ    594000
  #define DEMO_PIXEL_CLOCK_DIV   22
  #define DEMO_MIPI_DPY_REF_DIV  22

#elif (USE_MIPI_PANEL == MIPI_PANEL_HDMI_720X576_50)

  #define DEMO_PANEL_WIDTH  (720)
  #define DEMO_PANEL_HEIGHT (576)
  #define DEMO_PANEL_NAME   "HDMI_720X576_50"

  #define DEMO_HSW 64
  #define DEMO_HBP 68
  #define DEMO_HFP 12
  #define DEMO_VSW 5
  #define DEMO_VBP 39
  #define DEMO_VFP 5

  #define DEMO_VIDEO_FREQ_KHZ    594000
  #define DEMO_PIXEL_CLOCK_DIV   22
  #define DEMO_MIPI_DPY_REF_DIV  22

#elif (USE_MIPI_PANEL == MIPI_PANEL_HDMI_800X480_60)

  #define DEMO_PANEL_WIDTH  (800)
  #define DEMO_PANEL_HEIGHT (480)
  #define DEMO_PANEL_NAME   "HDMI_800X480_60"

  #define DEMO_HSW 20
  #define DEMO_HFP 70
  #define DEMO_HBP 26
  #define DEMO_VSW 2
  #define DEMO_VFP 7
  #define DEMO_VBP 21

  #define DEMO_VIDEO_FREQ_KHZ    594000
  #define DEMO_PIXEL_CLOCK_DIV   22
  #define DEMO_MIPI_DPY_REF_DIV  22

#elif (USE_MIPI_PANEL == MIPI_PANEL_HDMI_1024X600_76)

  #define DEMO_PANEL_WIDTH  (1024)
  #define DEMO_PANEL_HEIGHT (600)
  #define DEMO_PANEL_NAME   "HDMI_1024X600_76"

  #define DEMO_HSW 136
  #define DEMO_HBP 160
  #define DEMO_HFP 24
  #define DEMO_VSW 6
  #define DEMO_VBP 29
  #define DEMO_VFP 3

  #define DEMO_VIDEO_FREQ_KHZ    260000
  #define DEMO_PIXEL_CLOCK_DIV   4
  #define DEMO_MIPI_DPY_REF_DIV  10

#elif (USE_MIPI_PANEL == MIPI_PANEL_HDMI_1024X768_60)

  #define DEMO_PANEL_WIDTH  (1024)
  #define DEMO_PANEL_HEIGHT (768)
  #define DEMO_PANEL_NAME   "HDMI_1024X768_60"

  #define DEMO_HSW 136
  #define DEMO_HBP 160
  #define DEMO_HFP 24
  #define DEMO_VSW 6
  #define DEMO_VBP 29
  #define DEMO_VFP 3

  #define DEMO_VIDEO_FREQ_KHZ    260000
  #define DEMO_PIXEL_CLOCK_DIV   4
  #define DEMO_MIPI_DPY_REF_DIV  10

#elif (USE_MIPI_PANEL == MIPI_PANEL_HDMI_1280X720_60)

  #define DEMO_PANEL_WIDTH  (1280)
  #define DEMO_PANEL_HEIGHT (720)
  #define DEMO_PANEL_NAME   "HDMI_1280X720_60"

  #define DEMO_HSW 40
  #define DEMO_HBP 220
  #define DEMO_HFP 110
  #define DEMO_VSW 5
  #define DEMO_VBP 20
  #define DEMO_VFP 5

  #define DEMO_VIDEO_FREQ_KHZ    594000
  #define DEMO_PIXEL_CLOCK_DIV   8
  #define DEMO_MIPI_DPY_REF_DIV  22

#else
  #error Must set USE_MIPI_PANEL to one of the valid USE_MIPI_PANEL* values
#endif


#define IS_HDMI_PANEL()         (USE_MIPI_PANEL >= 10)
#define USE_FT5406_TOUCH()      ((USE_MIPI_PANEL == MIPI_PANEL_RK043FN02HC) || (MIPI_PANEL_NHD50800480TFATXL == USE_MIPI_PANEL))
#define USE_GT911_TOUCH()       ((MIPI_PANEL_RK055AHD091 == USE_MIPI_PANEL) || (MIPI_PANEL_RK055IQH091 == USE_MIPI_PANEL) || (MIPI_PANEL_RK055MHD091 == USE_MIPI_PANEL))
#define USE_USB_TOUCH()         (0)  /* Should be IS_HDMI_PANEL() if it worked */
#define IS_TOUCH_SUPPORTED()    (USE_FT5406_TOUCH() || USE_GT911_TOUCH() || USE_USB_TOUCH())
#define IS_PARALLEL_RGB_PANEL() ((USE_MIPI_PANEL == MIPI_PANEL_RK043FN02HC) || (USE_MIPI_PANEL == MIPI_PANEL_KD050WVTPA045) || (USE_MIPI_PANEL == MIPI_PANEL_NHD50800480TFATXL))


#if (USE_MIPI_PANEL == MIPI_PANEL_RK043FN02HC)
  #define DEMO_POL_FLAGS \
    (kELCDIF_DataEnableActiveHigh | kELCDIF_VsyncActiveLow | kELCDIF_HsyncActiveLow \
     | kELCDIF_DriveDataOnRisingClkEdge)
#else
  #define DEMO_POL_FLAGS \
    (kELCDIF_DataEnableActiveHigh | kELCDIF_VsyncActiveLow | kELCDIF_HsyncActiveLow \
     | kELCDIF_DriveDataOnFallingClkEdge)
#endif


//#define DEMO_PANEL_RK055AHD091 MIPI_PANEL_RK055AHD091
//#define DEMO_PANEL_RK055IQH091 MIPI_PANEL_RK055IQH091
//#define DEMO_PANEL_RK055MHD091 MIPI_PANEL_RK055MHD091

//#ifndef DEMO_PANEL
//  #define DEMO_PANEL USE_MIPI_PANEL
//#endif

#ifndef DEMO_DISPLAY_CONTROLLER
  /* Use LCDIFV2 by default, could use ELCDIF by changing this macro. */
  #define DEMO_DISPLAY_CONTROLLER DEMO_DISPLAY_CONTROLLER_LCDIFV2
#endif

#define DEMO_BUFFER_WIDTH   DEMO_PANEL_WIDTH
#define DEMO_BUFFER_HEIGHT  DEMO_PANEL_HEIGHT

/*
 * The DPHY bit clock must be fast enough to send out the pixels, it should be
 * larger than:
 *
 *         (Pixel clock * bit per output pixel) / number of MIPI data lane
 *
 * Here the desired DPHY bit clock multiplied by ( 9 / 8 = 1.125) to ensure
 * it is fast enough.
 */
#define DEMO_MIPI_DPHY_BIT_CLK_ENLARGE(origin) (((origin) / 8) * 9)

extern const dc_fb_t g_dc;
extern MIPI_DSI_Type g_mipiDsi;
#define DEMO_MIPI_DSI (&g_mipiDsi)
#define DEMO_MIPI_DSI_LANE_NUM 2

namespace Qul {
namespace Platform {
namespace Private {

void prepareDisplayController(void);
PlatformInterface::Screen *availableScreens(size_t *screenCount);

} // namespace Private
} // namespace Platform
} // namespace Qul
